/*
 * config.lex
 * 
 * $Revision: 15 $
 * $Date: 2006-07-22 16:46:05 +0200 (So, 22 čec 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
%{
#include <libgen.h>
#include <assert.h>
#include "config-lex.h"

#define DEBUG 0
#include "debug.h"

struct s_cfg cfg = {
	/* win_title */		PROG_NAME,
	/* max_widgets */	128,
	/* max_columns */	32,
	/* max_variables */	10000,
	/* win_width */		400,
	/* win_height */	100,
	/* layout_file */	"layout.conf",
	/* font */		{
	/*   FONT_NORMAL */	   "-adobe-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-2",
	/*   FONT_BIG */	   "-adobe-helvetica-bold-r-*-*-24-*-*-*-*-*-iso8859-2"
				}
};

#undef YY_DECL
#define YY_DECL static int config_lexscan()

int config_lex_wrap (void) {
	return 1;
}

enum { INT, STRING };

int *p_int = NULL;
char **p_str = NULL;

#define OPT(name, type) {						\
	p_int = NULL;							\
	p_str = NULL;							\
	if (type == INT) p_int = (int *) &(name);			\
	if (type == STRING) p_str = (char **) &(name);			\
	DBG_MSG(4, "Option %s (%s)", __STRING(name), __STRING(type));	\
	BEGIN(E);							\
}

static long line_number = 1;
%}
%s E
%s V
%s B
%%

max-widgets	OPT(cfg.max_widgets,		INT);
max-columns	OPT(cfg.max_columns,		INT);
max-variables	OPT(cfg.max_variables,		INT);
layout-file	OPT(cfg.layout_file,		STRING);
font-normal	OPT(cfg.font[FONT_NORMAL],	STRING);
font-bold	OPT(cfg.font[FONT_BOLD],	STRING);
font-big	OPT(cfg.font[FONT_BIG],		STRING);

<INITIAL>[a-zA-Z0-9_-]+ {	/* keyword*/
			BEGIN(B);
}

<E>[ \t]*=[ \t]* {	/* equal */
			BEGIN(V);
}
<E>[ \t]*[^=]+[ \t]*	BEGIN(B);

<B>[^\n]*		/* ignore rest of line in Bad state */
<B>\n			BEGIN(INITIAL);

<V>[+-]?[1-9][0-9]* {	/* decimal number */
			if (p_int) {
				*p_int = atoi(yytext);
				DBG_MSG(5, "Value: %d", *p_int);
			}
			BEGIN(INITIAL);
}

<V>\"(\\\"|[^\"])+\" {	/* string */
			if (p_str) {
				yytext[strlen(yytext) - 1] = 0;
				*p_str = strdup(yytext + 1);
				DBG_MSG(5, "Value: \"%s\"", *p_str);
			}
			BEGIN(INITIAL);
}

<V>[^\n]* {		/* unknown value */
			fprintf(stderr, PROG_NAME ": Unrecognized value \"%s\" at line %ld - ignored.\n", yytext, line_number);
			BEGIN(INITIAL);
}

[ \t]+			/* eat up whitespace */
\#[^\n]*		/* eat up comments */
\n			line_number++;

[^ \t\n]+		{
			fprintf(stderr, PROG_NAME ": Unrecognized word \"%s\" at line %ld\n", yytext, line_number);
			exit(1);
			}

%%

int config_load(char *filename) {
	char *d, *b;
	int r = 0;
	assert(filename != NULL);

	d = strdup(filename);
	b = strdup(filename);
	
	chdir(dirname(d));
	
	if ((config_lex_in = fopen(basename(b), "r"))) {
		config_lexscan();
		fclose(config_lex_in);
		r = 1;
	} else {
		perror(filename);
	}
	free(d);
	free(b);
	return r;
}

