/*
 * main.c -- icebuteo
 *
 * $Revision: 16 $
 * $Date: 2006-07-22 21:36:22 +0200 (Sat, 22 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>

#include "icebuteo.h"
#include "xwindow.h"
#include "vars.h"
#include "config-lex.h"

#define DEBUG 0
#include "debug.h"

int the_time = 0;

int main (int argc, char *argv[]) {
	int done = 0;
	XWINDOW *w;
	fd_set rd;
	struct timeval tv;
	int interval = 250e3;
	int s, l;
	char *str;
	
	memset(&tv, 0, sizeof(struct timeval));
	tv.tv_usec = interval;
	
	/* load configuration */
	if (argc == 2) {
		l = 0;
		str = argv[1];
	} else {
		l = strlen(getenv("HOME")) + strlen(DEFAULT_CONFIG) + 2;
		if (!(str = (char*) malloc(l))) {
			perror(PROG_NAME);
			abort();
		}
		strcpy(str, getenv("HOME"));
		strcat(str, "/");
		strcat(str, DEFAULT_CONFIG);
	}
	config_load(str);
	if (l)
		free(str);
	
	/* initialisation */
	
	var_init();
	w = xwindow_init();
	xwindow_load_widgets(w, "default-layout.conf");
	
	/* main loop */
	while (!done && !xwindow_is_broken(w)) {
		/* select init */
		FD_ZERO(&rd);
		FD_SET(xwindow_get_fd(w), &rd);
		
		s = select(FD_SETSIZE, &rd, NULL, NULL, &tv);
	
		if (s == -1) {
			/* error */
			perror(PROG_NAME ": select");
		} else if (s > 0) {
			/* data */
			if (FD_ISSET(xwindow_get_fd(w), &rd)) {
				/* X window event */
				xwindow_process_events(w);
			}
		} else {
			/* timeout */
			tv.tv_sec = 0;
			tv.tv_usec = interval;
			the_time += 1;
			var_set_as_float(112, (the_time % 100) / 100.0);
			xwindow_redraw(w, 0);
		}
	}
	
	return 0;
}

