/*
 * vars.c -- icebuteo
 * 
 * $Revision: 16 $
 * $Date: 2006-07-22 21:36:22 +0200 (Sat, 22 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "icebuteo.h"
#include "vars.h"

#define DEBUG 0
#include "debug.h"

typedef enum {
	VAR_EMPTY, VAR_INT, VAR_FLOAT, VAR_STRING
} VAR_TYPE;

typedef struct {
	VAR_TYPE type;
	long id;
	int i;
	float f;
	char *s;
} VAR;

static int variable_count = 0;
static VAR *variable = NULL;

int var_init(void) {
	assert(cfg.max_widgets > 0);
	assert(variable == NULL && variable_count == 0);
	
	DBG_MSG(4, "Initializing...");
	variable_count = cfg.max_variables + cfg.max_widgets + 2;
	if (!(variable = (VAR *)malloc(sizeof(VAR) * variable_count))) {
		perror(PROG_NAME ": var_init");
		exit(1);
	}
	memset(variable, 0, sizeof(VAR) * variable_count);
	DBG_MSG(4, "Initializing... Done.");
	return 1;
}


void var_destroy(void) {
	assert(variable != NULL);
	assert(variable_count > 0);

	free(variable);
	variable = NULL;
	variable_count = 0;
}


#define VAR_VR_RECALC	vr = vr < 0 ? vr + variable_count : vr

static void var_erase (int vr) {
	VAR_VR_RECALC;
	if (variable[vr].s)
		free(variable[vr].s);
	variable[vr].type = VAR_EMPTY;
}

void var_set_as_int (VAR_REF vr, int value) {
	VAR_VR_RECALC;
	if (variable[vr].type)
		var_erase(vr);
	variable[vr].type = VAR_INT;
	variable[vr].i = value;
	DBG_MSG(4, "variable[%d] = %d (INT)", vr, variable[vr].i);
}


void var_set_as_float (VAR_REF vr, float value) {
	VAR_VR_RECALC;
	if (variable[vr].type)
		var_erase(vr);
	variable[vr].type = VAR_FLOAT;
	variable[vr].f = value;
	DBG_MSG(4, "variable[%d] = %0.0f (FLOAT)", vr, variable[vr].f);
}


void var_set_as_string (VAR_REF vr, char* value) {
	VAR_VR_RECALC;
	if (variable[vr].type)
		var_erase(vr);
	variable[vr].type = VAR_STRING;
	variable[vr].s = strdup(value);
	DBG_MSG(4, "variable[%d] = \"%s\" (STRING)", vr, variable[vr].s);
}


int var_get_type (VAR_REF vr) {
	VAR_VR_RECALC;
	return variable[vr].type;
}


int var_get_as_int (VAR_REF vr) {
	DBG_MSG(4, "Reading variable[%d]...", vr);
	VAR_VR_RECALC;
	switch(variable[vr].type) {
		case VAR_INT:
			return variable[vr].i;
		case VAR_FLOAT:
			return (int)variable[vr].f;
		case VAR_STRING:
			return atoi(variable[vr].s);
		default:
			return 0;
	}
}


float var_get_as_float (VAR_REF vr) {
	DBG_MSG(4, "Reading variable[%d]...", vr);
	VAR_VR_RECALC;
	switch(variable[vr].type) {
		case VAR_INT:
			return (float)variable[vr].i;
		case VAR_FLOAT:
			return variable[vr].f;
		case VAR_STRING:
			return atof(variable[vr].s);
		default:
			return 0.0;
	}
}


char *var_get_as_string (VAR_REF vr) {
	DBG_MSG(4, "Reading variable[%d]...", vr);
	VAR_VR_RECALC;
	switch(variable[vr].type) {
		case VAR_INT:
			if (!variable[vr].s) {
				variable[vr].s = (char*)malloc(20);
				snprintf(variable[vr].s, 20, "%d", variable[vr].i);
			}
			return variable[vr].s;
		case VAR_FLOAT:
			if (!variable[vr].s) {
				variable[vr].s = (char*)malloc(20);
				snprintf(variable[vr].s, 20, "%1.3f", variable[vr].f);
			}
			return variable[vr].s;
		case VAR_STRING:
			return variable[vr].s;
		default:
			return NULL;
	}
}



