/*
 * xwindow.h -- icebuteo
 *
 * $Revision: 14 $
 * $Date: 2006-07-22 12:15:50 +0200 (Sat, 22 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#ifndef JK_XWINDOW_H
#define JK_XWINDOW_H

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "icebuteo.h"
#include "vars.h"

enum { 
	C_BG, C_DARK, C_FG, C_TEXT,
	C_SH_L, C_SH_D,
	C_BAR, C_BAR_L, C_BAR_S,
	C_BORDER,
	C_MAX
};

enum {
	WID_NONE,
	WID_FRAME,
	WID_VSEP,
	WID_HSEP,
	WID_TEXT,
	WID_TEXT_DEFAULT,
	WID_TEXT_FONT,
	WID_TEXT_ALIGN,
	WID_TEXT_SHADOW,
	WID_BAR
};

typedef struct {
	int type;			/* widget type */
	int column;			/* column id */
	int x, y;			/* position in column */
	int width, height;		/* size */
	VAR_REF var_ref;		/* showed value */
} WIDGET;

typedef struct {
	int not_empty;			/* there is something in column */
	VAR_REF visibility_var_id;	/* if zero, don't show this */
	int width, height;		/* wanted size */
	int x, y;			/* position -- automaticaly updated */
	int visible;			/* column is visible -- automaticaly updated */
} COLUMN;

typedef struct {
	/* about window */
	Display *disp;
	Window win;
	GC gc;
	int x, y, width, height;
	int root_width, root_height;
	XFontStruct *font[FONT_MAX];
	int act_fid;
	int can_redraw, is_broken, is_font_selected;
	XColor color[C_MAX];

	/* about widgets */
	WIDGET *widget;
	COLUMN *column;
	int widget_count, column_count;
} XWINDOW;



XWINDOW * xwindow_init(void);
void xwindow_load_widgets(XWINDOW *xwin, char *filename);
void xwindow_destroy(XWINDOW *xwin);
void xwindow_process_events(XWINDOW *xwin);
void xwindow_reposition (XWINDOW *xwin, int now);
void xwindow_recalculate(XWINDOW *xwin);
void xwindow_redraw(XWINDOW *xwin, int recalculate);

#define xwindow_get_fd(xwin) {ConnectionNumber((xwin)->disp);}
#define xwindow_is_broken(xwin) ((xwin)->is_broken)

#endif

