/*
 * xwindow.lex
 * 
 * $Revision: 17 $
 * $Date: 2006-07-23 13:23:39 +0200 (Sun, 23 Jul 2006) $
 * 
 * Copyright (C) 2006  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
%{
#include "xwindow-lex.h"
#include "xdraw.h"

#define DEBUG 0
#include "debug.h"

#undef YY_DECL
#define YY_DECL int xwindow_lexscan(XWINDOW *xwin)

int xwindow_lex_wrap (void) {
	return 1;
}

#define WID(t) {					\
	c = 0;						\
	wid = &(xwin->widget[widget_i++]); 		\
	col = NULL;					\
	wid->type = t;					\
	dim[0] = &(wid->column);			\
	dim[1] = &(wid->x);				\
	dim[2] = &(wid->y);				\
	dim[3] = &(wid->width);				\
	dim[4] = &(wid->height);			\
	DBG_MSG(5, "Loading \"%s\"", __STRING(t));	\
}

static long line_number = 1;
static int widget_i = 0;
static int column_i = 0;
static int c = 0;
static WIDGET *wid = NULL;
static COLUMN *col = NULL;
static int *dim[5] = { NULL, NULL, NULL, NULL, NULL };
static int var_id = 0;
%}
%s W
%s WE
%s C
%s CE
%%

<C,W>\n {
			fprintf(stderr, PROG_NAME ": Missing data at line %ld\n", line_number);
			exit(1);
}

<WE>\n {		/* konec radku s widgetem */
			if (wid && wid->type != WID_NONE) {
				xwin->column[wid->column].not_empty = 1;
				DBG_MSG(4, "Loaded widget %d: type %d at %d, %d", widget_i - 1, wid->type, wid->x, wid->y);
			}
			line_number++;
			BEGIN(INITIAL);
}
			

<CE>\n {		/* konec radku se sloupcem */
			if (col) {
				DBG_MSG(4, "Loaded column %d: %d x %d", column_i-1, col->width, col->height);
			}
			line_number++;
			BEGIN(INITIAL);
}

\#[^\n]*		/* eat up comments */
[ \t]+			/* eat up whitespace */
\n			line_number++;

COLUMN {		/* zacatek radku se sloupcem */
			wid = NULL;
			col = &(xwin->column[column_i]);
			c = 0;
			dim[0] = &(xwin->column[column_i].width);
			dim[1] = &(xwin->column[column_i].height);
			DBG_MSG(5, "Loading column %d...", column_i);
			column_i++;
			BEGIN(C);
}

NONE			WID(WID_NONE);         BEGIN(W);
FRAME			WID(WID_FRAME);        BEGIN(W);
VSEP			WID(WID_VSEP);         BEGIN(W);
HSEP			WID(WID_HSEP);         BEGIN(W);
TEXT			WID(WID_TEXT);         BEGIN(W);
TEXT_DEFAULT		WID(WID_TEXT_DEFAULT); BEGIN(W);
TEXT_FONT		WID(WID_TEXT_FONT);    BEGIN(W);
TEXT_ALIGN		WID(WID_TEXT_ALIGN);   BEGIN(W);
TEXT_SHADOW		WID(WID_TEXT_SHADOW);  BEGIN(W);
BAR			WID(WID_BAR);          BEGIN(W);

<C>[0-9]+ {		/* integer pro column */
			if (c >= 0 && c < 2 && dim[c]) {
				*(dim[c]) = atoi(yytext);
				DBG_MSG(6, "Integer: col = %d, value = %d", c, *(dim[c]));
			} else if (c == 2) {
				var_id = atoi(yytext);
				DBG_MSG(6, "Integer: col = %d, var_id = %d", c, var_id);
			} else if (c == 3) {
				col->visibility_var_id = var_id;
				var_set_as_int(var_id, atoi(yytext));
				DBG_MSG(6, "Integer: col = %d, var[%d] = %d", c, var_id, 
						var_get_as_int(var_id));
				BEGIN(CE);
			}
			c++;
}


<W>\"(\\\"|[^\"])+\" {	/* string pro widget */
			if (c == 6) {
				wid->var_ref = var_id;
				yytext[strlen(yytext) - 1] = 0;
				var_set_as_string(var_id, yytext + 1);
				DBG_MSG(6, "String: col = %d, var[%d] = \"%s\"", c, var_id, 
						var_get_as_string(var_id));
				BEGIN(WE);
			}
}

<W>[+-]?([0-9]*\.[0-9]+|[0-9]+\.[0-9]*) {	/* float pro widget */
			if (c == 6) {
				wid->var_ref = var_id;
				var_set_as_float(var_id, atof(yytext));
				DBG_MSG(6, "String: col = %d, var[%d] = %0.0f", c, var_id, 
						var_get_as_float(var_id));
				BEGIN(WE);
			}
}

<W>[+-]?[0-9]+ {	/* integer pro widget */
			if (c >= 0 && c < 5 && dim[c]) {
				*(dim[c]) = atoi(yytext);
				DBG_MSG(6, "Integer: col = %d, value = %d", c, *(dim[c]));
			} else if (c == 5) {
				var_id = atoi(yytext);
				DBG_MSG(6, "Integer: col = %d, var_id = %d", c, var_id);
			} else if (c == 6) {
				wid->var_ref = var_id;
				var_set_as_int(var_id, atoi(yytext));
				DBG_MSG(6, "Integer: col = %d, var[%d] = %d", c, var_id, 
						var_get_as_int(var_id));
				BEGIN(WE);
			}
			c++;
}

<W>- {			/* alias pro nulu */
			if (c >= 0 && c < 5 && dim[c]) {
				*(dim[c]) = 0;
			} else if (c == 5) {
				var_id = - widget_i;
				DBG_MSG(6, "Integer: col = %d, var_id = %d", c, var_id);
			} else if (c == 6) {
				wid->var_ref = var_id;
				var_set_as_int(var_id, atoi(yytext));
				DBG_MSG(6, "Integer: col = %d, var[%d] = %d", c, var_id, 
						var_get_as_int(var_id));
				BEGIN(WE);
			}
			c++;
}


[^ \t\n]+		{
			fprintf(stderr, PROG_NAME ": Unrecognized word \"%s\" at line %ld\n", yytext, line_number);
			exit(1);
			}

%%

